@@ -21,6 +21,12 @@ Catalyst::Test - Test Catalyst Applications
     request('index.html');
     get('index.html');
 
+    use HTTP::Request::Common;
+    my $response = request POST '/foo', [
+        bar => 'baz',
+        something => 'else'
+    ];
+
     # Run tests against a remote server
     CATALYST_SERVER='http://localhost:3000/' prove -r -l lib/ t/
 
@@ -45,7 +51,13 @@ Catalyst::Test - Test Catalyst Applications
 
 =head1 DESCRIPTION
 
-Test Catalyst Applications.
+This module allows you to make requests to a Catalyst application either without
+a server, by simulating the environment of an HTTP request using
+L<HTTP::Request::AsCGI> or remotely if you define the CATALYST_SERVER
+environment variable.
+
+The </get> and </request> functions take either a URI or an L<HTTP::Request>
+object.
 
 =head2 METHODS
 
@@ -55,6 +67,17 @@ Returns the content.
 
     my $content = get('foo/bar?test=1');
 
+Note that this method doesn't follow redirects, so to test for a
+correctly redirecting page you'll need to use a combination of this
+method and the L<request> method below:
+
+    my $res = request('/'); # redirects to /y
+    warn $res->header('location');
+    use URI;
+    my $uri = URI->new($res->header('location'));
+    is ( $uri->path , '/y');
+    my $content = get($uri->path);
+
 =head2 request
 
 Returns a C<HTTP::Response> object.
@@ -72,9 +95,10 @@ sub import {
     if ( $ENV{CATALYST_SERVER} ) {
         $request = sub { remote_request(@_) };
         $get     = sub { remote_request(@_)->content };
-    }
-
-    else {
+    } elsif (! $class) {
+        $request = sub { Catalyst::Exception->throw("Must specify a test app: use Catalyst::Test 'TestApp'") };
+        $get     = $request;
+    } else {
         unless( Class::Inspector->loaded( $class ) ) {
             require Class::Inspector->filename( $class );
         }
@@ -92,6 +116,8 @@ sub import {
 
 =head2 local_request
 
+Simulate a request using L<HTTP::Request::AsCGI>.
+
 =cut
 
 sub local_request {
@@ -123,7 +149,31 @@ sub remote_request {
     my $server  = URI->new( $ENV{CATALYST_SERVER} );
 
     if ( $server->path =~ m|^(.+)?/$| ) {
-        $server->path("$1");    # need to be quoted
+        my $path = $1;
+        $server->path("$path") if $path;    # need to be quoted
+    }
+
+    # the request path needs to be sanitised if $server is using a
+    # non-root path due to potential overlap between request path and
+    # response path.
+    if ($server->path) {
+        # If request path is '/', we have to add a trailing slash to the
+        # final request URI
+        my $add_trailing = $request->uri->path eq '/';
+        
+        my @sp = split '/', $server->path;
+        my @rp = split '/', $request->uri->path;
+        shift @sp;shift @rp; # leading /
+        if (@rp) {
+            foreach my $sp (@sp) {
+                $sp eq $rp[0] ? shift @rp : last
+            }
+        }
+        $request->uri->path(join '/', @rp);
+        
+        if ( $add_trailing ) {
+            $request->uri->path( $request->uri->path . '/' );
+        }
     }
 
     $request->uri->scheme( $server->scheme );
@@ -147,11 +197,12 @@ sub remote_request {
 
 =head1 SEE ALSO
 
-L<Catalyst>.
+L<Catalyst>, L<Test::WWW::Mechanize::Catalyst>,
+L<Test::WWW::Selenium::Catalyst>, L<Test::More>, L<HTTP::Request::Common>
 
-=head1 AUTHOR
+=head1 AUTHORS
 
-Sebastian Riedel, C<sri@cpan.org>
+Catalyst Contributors, see Catalyst.pm
 
 =head1 COPYRIGHT
 
